#
#    RedEye is a Disc Authoring and Burning program.
#    Copyright (C) 2003  Michael Urman
#
#    RedEye is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    RedEye is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with RedEye; if not, write to the Free Software Foundation, Inc., 59
#    Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

__version__ = '0.0.20'

import os, gtk

class RedEye(object):

    """RedEye.Application.RedEye

    Encapsulate the RedEye Application Interface.  This includes access to
    preferences, SDI Window generation, and more."""

    layouts = []
    devices = []
    verifiers = []

    def __init__(self):
        # set the window icon
        iconpath = Config.findicon('icon-eye.png')
        if os.path.exists(iconpath):
            pixbuf = gtk.gdk.pixbuf_new_from_file(iconpath)
            gtk.window_set_default_icon_list((pixbuf))

        Config.render_icons()

        # find devices
        if len(self.devices) == 0:
            try: self.devices.extend(CDRecord().get_devices())
            except ValueError: pass

        # find verification
        if len(self.verifiers) == 0:
            try: self.verifiers.extend(Mount().get_devices())
            except ValueError: pass

        RedEye.helpers = Dialogs.HelperLog()
        RedEye.messages = Dialogs.Messages()
        RedEye.preferences = Dialogs.Preferences()

        self.layouts.append(LayoutWindow())
        self.layouts[-1].show()

    def run(self):
        gtk.mainloop()

class LayoutWindow(gtk.Window):

    """RedEye.Application.LayoutWindow

    Each Disc you author (or load an image for, once that's supported) gets its
    own LayoutWindow in standard GNOME SDI fashion.  LayoutWindow encapsulates
    the gruntwork."""

    last_dir = "./"

    def __init__(self, layout=None):
        """LayoutWindow(layout=None)

        Create a RedEye Layout Window with layout type as specified, defaulting
        to Virtual"""

        gtk.Window.__init__(self)

        # main window
        self.set_title("Redeye %s" % __version__)
        self.set_default_size(gtk.gdk.screen_width()/3,
                gtk.gdk.screen_height()/2)
        self.connect('delete-event', self.on_delete)
        self.connect('destroy-event', self.on_destroy)

        # packed all in a vbox
        vbox = gtk.VBox()
        self.add(vbox)

        # Menus
        menubar = gtk.MenuBar()
        vbox.pack_start(menubar, expand=False)
        menus = [menubar]
        menuinfo = [Config.menuitems[:]]
        while 1:
            try:
                name, icon, attr = menuinfo[-1].pop(0)
            except IndexError:
                try:
                    menuinfo.pop()
                    menus.pop()
                except IndexError: break
                else: continue

            if name == '-':
                item = gtk.SeparatorMenuItem()
                item.show()
                menus[-1].append(item)
            else:
                if icon is bool:
                    item = gtk.CheckMenuItem(name)
                    item.set_active(True)
                else:
                    item = gtk.ImageMenuItem(name)
                    if icon and icon != name:
                        icon = gtk.image_new_from_stock(icon,gtk.ICON_SIZE_MENU)
                        item.set_image(icon)
                if name == '_Help':
                    item.set_right_justified(True)

                menus[-1].append(item)
                item.show()
                if isinstance(attr, list):
                    submenu = gtk.Menu()
                    item.set_submenu(submenu)
                    menus.append(submenu)
                    menuinfo.append(attr[:])
                else:
                    item.connect('activate', getattr(self, 'on_activate_'+attr))


        # Toolbar Buttons
        toolbar = self.toolbar = gtk.Toolbar()
        vbox.pack_start(toolbar, expand=False)
        toolbar.set_style(gtk.TOOLBAR_ICONS)
        toolbar.set_tooltips(True)
        for attr, text, icon, tooltip in Config.toolbaritems:
            if attr == '-':
                toolbar.append_space()
                continue
            callback = getattr(self, 'on_click_%s' % attr)
            privtip = tooltip.join('()')
            if isinstance(icon, basestring):
                icon = Config.pixbuf[icon][1]

            button = toolbar.append_item(text, tooltip, privtip, icon, callback)
            setattr(self, 'button_%s' % attr, button)

        # Disc Layout
        if layout is None:
            layout = Layout.Virtual.VirtualLayout()
        self.layout = layout

        swin = gtk.ScrolledWindow()
        swin.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_ALWAYS)
        swin.add(layout)
        swin.set_shadow_type(gtk.SHADOW_IN)
        vbox.pack_start(swin)

        # Progress Feedback
        hbox = gtk.HBox()
        vbox.pack_start(hbox, expand=False)

        self.create_progress = gtk.ProgressBar()
        self.create_progress.set_text("create: idle")
        self.fifo_fill = gtk.ProgressBar()
        self.fifo_fill.set_text("fifo: 0%")
        self.buffer_fill = gtk.ProgressBar()
        self.buffer_fill.set_text("buffer: 0%")

        hbox.pack_start(self.create_progress)
        hbox.pack_start(self.fifo_fill)
        hbox.pack_start(self.buffer_fill)

        self.main_progress = gtk.ProgressBar()
        self.main_progress.set_text("Idle")

        vbox.pack_start(self.main_progress, expand=False)

        # Status Feedback
        hbox = self.status = gtk.HBox()
        vbox.pack_start(hbox, expand=False)
        statusbar = self.layout.statusbar = gtk.Statusbar()
        statusbar.set_has_resize_grip(False)
        hbox.pack_start(statusbar)

        # Funky Buttons - HIG will probably nix these
        self.helper_log_button = RedEye.helpers.make_button()
        hbox.pack_start(self.helper_log_button, expand=False, fill=False)

        self.messages_button = RedEye.messages.make_button()
        hbox.pack_start(self.messages_button, expand=False, fill=False)

        vbox.show_all()

    def on_delete(self, *args):
        gtk.mainquit()

    def on_destroy(self, *args):
        gtk.mainquit()

    def on_click_error(self, button): RedEye.messages.error('Test Error')
    def on_click_warn(self, button): RedEye.messages.warn('Test Warn')
    def on_click_info(self, button): RedEye.messages.info('Test Info')

    def on_click_add_session(self, button): self.layout.append_session()
    def on_click_add_audio_track(self, button):
        self.add_audio_track()
    def on_click_add_data_track(self, button):
        self.layout.append_track(Layout.Virtual.VirtualDataTrack)
    def on_click_add_dir(self, button): self.layout.append_folder()
    def on_click_add_files(self, button): self.add_file()
    def on_click_delete_selected(self, button): self.layout.remove_selected()
    def on_click_prefs(self, button): pass
    def on_click_burn(self, button): pass

    def on_activate_open(self, menuitem):
        """Handle New,Open,Revert"""
        action = menuitem.get_child().get_text()
        print action

    def on_activate_save(self, menuitem):
        """Handle Save,Save As"""
        action = menuitem.get_child().get_text()
        print action

    def on_activate_close(self, menuitem):
        """Handle Close,Quit"""
        action = menuitem.get_child().get_text()
        print action

    def on_activate_edit(self, menuitem):
        """Handle Undo,Redo,Cut,Copy,Paste,Delete"""
        action = menuitem.get_child().get_text()
        if action == 'Delete':
            self.layout.remove_selected()
        print action

    def on_activate_pref_prop(self, menuitem):
        """Handle Preferences,Properties"""
        action = menuitem.get_child().get_text()
        if action == 'Preferences':
            RedEye.preferences.run()
            RedEye.preferences.hide()
        else:
            print action

    def on_activate_view(self, menuitem):
        """Handle Toolbar,Statusbar,Messages,Output"""
        action = menuitem.get_child().get_text()
        if action == 'Toolbar':
            if menuitem.get_active(): self.toolbar.show()
            else: self.toolbar.hide()
        elif action == 'Statusbar':
            if menuitem.get_active(): self.status.show()
            else: self.status.hide()
        elif action == 'Messages...':
            RedEye.messages.present()
        elif action == 'Output Logs...':
            RedEye.helpers.present()
        else:
            raise RuntimeError("Unexpected Menu Selection '%s'" % action)

    def on_activate_insert(self, menuitem):
        """Handle Session,Audio Track,Data Track,Folder,File"""
        action = menuitem.get_child().get_text()
        if action == 'Session':
            self.layout.append_session()
        elif action == 'Audio Track':
            self.add_audio_track()
        elif action == 'Data Track':
            self.layout.append_track(Layout.Virtual.VirtualDataTrack)
        elif action == 'Folder':
            self.layout.append_folder()
        elif action == 'File':
            self.add_file()
        else:
            raise RuntimeError("Unexpected Menu Selection '%s'" % action)

    def choose_file(self, title=None):
        filesel = gtk.FileSelection(title)
        filesel.hide_fileop_buttons()
        filesel.set_select_multiple(True)
        filesel.complete(os.path.join(self.last_dir)+os.sep)
        status = filesel.run()
        filesel.hide()
        if status == gtk.RESPONSE_OK:
            files = filesel.get_selections()
            self.last_dir = os.path.dirname(files[0])
        else:
            files = tuple()
        filesel.destroy()
        return map(os.path.abspath, files)

    def add_file(self):
        layout = self.layout
        files = self.choose_file('Choose File(s) or Directory...')
        for file in files:
            name = os.path.basename(os.path.normpath(file))
            if os.path.isdir(file):
                layout.append_file(
                    Layout.Virtual.VirtualFolder(name, layout, file))
            else:
                layout.append_file(
                    Layout.Virtual.VirtualFile(name, layout, file))

    def add_audio_track(self):
        layout = self.layout
        files = self.choose_file('Choose Audio File(s)...')
        for file in files:
            name = os.path.basename(os.path.normpath(file))
            if not os.path.isdir(file):
                layout.append_track(
                    Layout.Virtual.VirtualAudioTrack(name, layout, file))

    def on_activate_help(self, menuitem):
        """Handle Help,About"""
        action = menuitem.get_child().get_text()
        print action

    def on_activate_describe(self, menuitem):
        self.layout.describe()

# Need these at runtime, but they might need RedEye.Appliation.RedEye to be
# defined first
import Config, Dialogs, Layout.Virtual
from Helper.Cdrecord import CDRecord
from Helper.Mkisofs import Mkisofs
from Helper.Mount import Mount
