#
#    RedEye is a Disc Authoring and Burning program.
#    Copyright (C) 2003  Michael Urman
#
#    RedEye is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    RedEye is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with RedEye; if not, write to the Free Software Foundation, Inc., 59
#    Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

from RedEye.Application import RedEye
import gtk, time

class HelperLog(gtk.Dialog):
    
    """RedEye.Dialogs.HelperLog

    Dialog that tracks output from Helpers that otherwise goes unparsed and
    unreported"""

    RESPONSE_CLEAR = 1

    def __init__(self):
        gtk.Dialog.__init__(self, title="RedEye: Helper Output Log")
        self.set_default_size(gtk.gdk.screen_width()/4,
                gtk.gdk.screen_height()/3)
        self.notebook = gtk.Notebook()
        self.vbox.pack_start(self.notebook)
        self.vbox.show_all()

        self.views = {}

        # Buttons
        self.add_button('gtk-clear', self.RESPONSE_CLEAR)
        self.add_button('gtk-close', gtk.RESPONSE_CLOSE)
        self.set_has_separator(False)

        self.connect('response', self.on_response)
        self.connect('close', self.on_close)
        self.connect('delete-event', self.on_close)

    def on_response(self, dialog, response):
        if response in (gtk.RESPONSE_DELETE_EVENT, gtk.RESPONSE_CLOSE):
            self.hide()
        elif response == dialog.RESPONSE_CLEAR:
            # Clear the current notebook page (HMM: clear all?)
            notebook = self.notebook
            n = notebook.get_current_page()
            if n < 0:
                return True
            swin = notebook.get_nth_page(n)
            title = notebook.get_tab_label_text(swin)

            view = self.view[title]
            buf = view.get_buffer()
            buf.delete(buf.get_bounds())
        else:
            RedEye.messages.error('InternalError: Unhandled messages dialog response %d' % response)
        return True

    def on_close(self, dialog, event=None):
        self.hide()
        return True # don't carry on to potentially destroy; JUST hide

    def program_tab(self, title):
        """HelperLog.program_tab(self, title)

        Retrieve or create and retrieve the view for program <title>"""

        try:
            return self.view[title]
        except KeyError:
            view = gtk.TextView()
            self.view[title] = view
            view.set_editable(False)
            view.set_cursor_visible(False)
            buf = view.get_buffer()
            table = buf.get_tag_table()

            out_tag = gtk.TextTag('stdout')
            out_tag.set_property('foreground', 'blue')
            table.add(out_tag)

            err_tag = gtk.TextTag('stderr')
            err_tag.set_property('foreground', 'black')
            table.add(err_tag)

            head_tag = gtk.TextTag('header')
            head_tag.set_property('scale', pango.SCALE_LARGE)
            head_tag.set_property('weight', pango.WEIGHT_BOLD)
            table.add(head_tag)

            swin = gtk.ScrolledWindow()
            swin.add(view)

            label = gtk.Label(title)
            self.notebook.append_page(swin, label)

            swin.show_all()
            return view

    def log(self, view, message, tag):
        """HelperLog.log(self, view, message, tag)

        Log <message> to <view> using <tag>"""

        message = message.strip()
        message = message.decode('iso-8859-1', 'replace').encode('utf-8')+'\n'
        buf = view.get_buffer()
        sob, eob = buf.get_bounds()
        buf.insert_with_tags_by_name(eob, message, tag)
        view.scroll_to_mark(buf.get_insert(), 0)

    def make_button(self):
        button = gtk.Button()
        button.set_relief(gtk.RELIEF_NONE)
        icon = gtk.image_new_from_stock('gtk-justify-left', gtk.ICON_SIZE_MENU)
        button.add(icon)
        button.connect('clicked', lambda button: self.present())
        return button

class Messages(gtk.Dialog):

    """RedEye.Dialogs.Messages

    Dialog that tracks times and severities of messages.  Severity levels are
    Info, Warn, and Error."""

    buttons = []
    RESPONSE_CLEAR = 1

    INFO, WARN, ERROR = range(3)
    current_level = INFO

    def __init__(self):
        gtk.Dialog.__init__(self, title="RedEye: Messages")
        self.set_default_size(400,300)

        label = gtk.Label('Messages:')
        label.set_padding(5, 3)
        label.set_alignment(0, 0)
        self.vbox.pack_start(label, expand=False)

        self.model = gtk.ListStore(gtk.gdk.Pixbuf, str, float)
        self.view = gtk.TreeView(self.model)
        self.view.set_rules_hint(True)
        self.view.set_headers_visible(False)
        self.view.append_column(
                gtk.TreeViewColumn('', gtk.CellRendererPixbuf(), pixbuf=0))
        self.view.append_column(
                gtk.TreeViewColumn('Time', gtk.CellRendererText(), text=2))
        self.view.append_column(
                gtk.TreeViewColumn('Message', gtk.CellRendererText(), text=1))

        # Turn the float column into HH:MM:SS time
        def render_hms(column, cell, model, iter):
            renderer = column.get_cell_renderers()[0]
            msg_time = model.get_value(iter, 2)
            renderer.set_property('text', time.strftime('%H:%M:%S',
                time.localtime(msg_time)))
        col = self.view.get_column(1)
        renderer = col.get_cell_renderers()[0]
        col.set_cell_data_func(renderer, render_hms)

        swin = gtk.ScrolledWindow()
        swin.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        swin.add(self.view)
        
        self.add_button('gtk-clear', self.RESPONSE_CLEAR)
        self.add_button('gtk-close', gtk.RESPONSE_CLOSE)

        self.vbox.pack_start(swin)
        self.vbox.show_all()
        self.set_has_separator(False)

        self.connect('response', self.on_response)
        self.connect('close', self.on_close)
        self.connect('delete-event', self.on_close)

    def on_response(self, dialog, response):
        if response in (gtk.RESPONSE_DELETE_EVENT, gtk.RESPONSE_CLOSE):
            self.set_icon(self.INFO, True)
            self.hide()
        elif response == self.RESPONSE_CLEAR:
            self.clear_messages()
        else:
            RedEye.messages.error('InternalError: Unhandled messages dialog response %d' % response)
        return True

    def on_close():
        self.hide()
        return True

    def make_button(self):
        button = gtk.Button()
        self.buttons.append(button) # track buttons for updates
        button.set_relief(gtk.RELIEF_NONE)
        icon = gtk.image_new_from_stock('gtk-dialog-info', gtk.ICON_SIZE_MENU)
        self.icon_info = icon
        icon.show()
        button.add(icon)
        icon = gtk.image_new_from_stock('gtk-dialog-warning',gtk.ICON_SIZE_MENU)
        self.icon_warn = icon
        icon.show()
        icon = gtk.image_new_from_stock('gtk-dialog-error', gtk.ICON_SIZE_MENU)
        self.icon_error = icon
        icon.show()
        button.connect('clicked', self.on_button_clicked)

        return button

    def on_button_clicked(self, button):
        self.present()

    def present(self):
        self.set_icon(self.INFO, True)
        gtk.Dialog.present(self)

    def clear_messages(self):
        self.set_icon(self.INFO, True)
        while 1:
            it = self.model.get_iter_first()
            if not it: break
            self.model.remove(it)

    def set_icon(self, level, force=False):
        if level > self.current_level or force:
            self.current_level = level
            if level == self.INFO: icon = self.icon_info
            elif level == self.WARN: icon = self.icon_warn
            elif level == self.ERROR: icon = self.icon_error
            else: raise ValueError("Invalid messages level")
        else:
            icon = None
            level = self.current_level

        if force: relief = gtk.RELIEF_NONE
        else: relief = gtk.RELIEF_NORMAL

        for button in self.buttons:
            button.set_relief(relief)
            if icon is not None:
                button.remove(button.get_child())
                button.add(icon)

    def message(self, message, level):
        if level == self.INFO: stock = 'gtk-dialog-info'
        elif level == self.WARN: stock = 'gtk-dialog-warning'
        elif level == self.ERROR: stock = 'gtk-dialog-error'
        else: raise ValueError("Invalid messages level")

        pixbuf = self.render_icon(stock_id=stock, size=gtk.ICON_SIZE_MENU)
        self.model.prepend((pixbuf, message, time.time()))

    def popup(self, message, level):
        if level == self.INFO: kind = gtk.MESSAGE_INFO
        elif level == self.WARN: kind = gtk.MESSAGE_WARNING
        elif level == self.ERROR: kind = gtk.MESSAGE_ERROR
        else: raise ValueError("Invalid messages level")

        dialog = gtk.MessageDialog(type=kind, buttons=gtk.BUTTONS_OK,
                message_format=message)
        dialog.set_response_sensitive(gtk.RESPONSE_OK, True)
        dialog.run()
        dialog.hide()
        dialog.destroy()
        del dialog

    def info(self, message, popup=False):
        self.set_icon(self.INFO)
        self.message(message, self.INFO)
        if popup: self.popup(message, self.INFO)
    def warn(self, message, popup=False):
        self.set_icon(self.WARN)
        self.message(message, self.WARN)
        if popup: self.popup(message, self.WARN)
    def error(self, message, popup=False):
        self.set_icon(self.ERROR)
        self.message(message, self.ERROR)
        if popup: self.popup(message, self.ERROR)

class Preferences(gtk.Dialog):

    """RedEye.Application.PreferencesDialog

    Select a writing device, device options, etc."""

    def __init__(self):
        gtk.Dialog.__init__(self, title="RedEye Preferences", buttons=('gtk-close', gtk.RESPONSE_CLOSE))

        hbox = gtk.HBox()
        self.vbox.pack_start(hbox, padding=12)
        Vbox = gtk.VBox()
        hbox.pack_start(Vbox, padding=12)

        label = gtk.Label()
        label.set_markup('<b>Writing Device:</b>')
        label.set_alignment(0.0, 0.5)
        Vbox.pack_start(label, expand=False)

        hbox = gtk.HBox()
        Vbox.pack_start(hbox, expand=False)
        hbox.pack_start(gtk.Label("    "), expand=False)
        vbox = gtk.VBox()
        hbox.pack_start(vbox)

        self.device = None
        self.verify = None
        self.opts = {}

        nodev = radio = gtk.RadioButton(label='None', group=None)
        radio.connect('toggled', self.on_device_toggled, None)
        vbox.pack_start(radio, expand=False)
        for dev in RedEye.devices:
            radio = gtk.RadioButton(label=dev.description, group=radio)
            radio.connect('toggled', self.on_device_toggled, dev)
            vbox.pack_start(radio, expand=False)
            if not dev.caps.get('writes_cdr'):
                radio.set_sensitive(False)

        label = gtk.Label()
        label.set_markup('<b>Writing Speed:</b>')
        label.set_alignment(0.0, 0.5)
        Vbox.pack_start(label, expand=False)

        hbox = gtk.HBox()
        Vbox.pack_start(hbox, expand=False)
        hbox.pack_start(gtk.Label("    "), expand=False)
        vbox = gtk.VBox()
        hbox.pack_start(vbox)

        radio = gtk.RadioButton(label="Maximum Supported", group=None)
        radio.connect('toggled', self.on_speed_toggled, False)
        vbox.pack_start(radio, expand=False)
        self.opts['Max Speed'] = [radio, True]

        hbox = gtk.HBox()
        vbox.pack_start(hbox, expand=False)
        radio = gtk.RadioButton(label="Custom: ", group=radio)
        radio.connect('toggled', self.on_speed_toggled, True)
        self.opts['Custom'] = [radio, False]
        hbox.pack_start(radio, expand=False)
        speed = gtk.SpinButton()
        speed.set_range(1, 52)
        speed.set_increments(1, 8)
        self.opts['Custom Speed'] = [speed, 1]
        speed.connect('value-changed', self.on_speed_changed)
        hbox.pack_start(speed, expand=False)

        label = gtk.Label()
        label.set_markup('<b>Writing Options:</b>')
        label.set_alignment(0.0, 0.5)
        Vbox.pack_start(label, expand=False)

        hbox = gtk.HBox()
        Vbox.pack_start(hbox, expand=False)
        hbox.pack_start(gtk.Label("    "), expand=False)
        vbox = gtk.VBox()
        hbox.pack_start(vbox)

        for text in ('Simulate Write', 'Burn Proof', 'Session At Once', 'Overburn'):
            check = gtk.CheckButton(text)
            check.connect('toggled', self.on_option_toggled, text)
            self.opts[text] = [check, False]
            vbox.pack_start(check, expand=False)


        label = gtk.Label()
        label.set_markup('<b>Verifying Device:</b>')
        label.set_alignment(0.0, 0.5)
        Vbox.pack_start(label, expand=False)

        hbox = gtk.HBox()
        Vbox.pack_start(hbox, expand=False)
        hbox.pack_start(gtk.Label("    "), expand=False)
        vbox = gtk.VBox()
        hbox.pack_start(vbox)

        novfy = radio = gtk.RadioButton(label='None', group=None)
        radio.connect('toggled', self.on_verify_toggled, None)
        vbox.pack_start(radio, expand=False)
        for vfy in RedEye.verifiers:
            radio = gtk.RadioButton(label=vfy.description, group=radio)
            radio.connect('toggled', self.on_verify_toggled, dev)
            vbox.pack_start(radio, expand=False)


        Vbox.set_spacing(6)
        self.set_has_separator(False)
        self.set_resizable(False)
        self.vbox.show_all()

        self.on_device_toggled(nodev, None)
        self.on_verify_toggled(novfy, None)

    def on_device_toggled(self, toggle, device):
        if toggle.get_active():
            self.device = device
            for cap, opt in [
                ('supports_test', 'Simulate Write'),
                ('supports_burnfree', 'Burn Proof'),
                ('', 'Session At Once'),
                ('', 'Overburn'),
                ('', 'Max Speed'),
                ('', 'Custom'),
            ]:
                self.opts[opt][0].set_sensitive(device is not None
                        and (cap == '' or device.caps.get(cap)))
            self.opts['Custom Speed'][0].set_editable(device is not None
                    and (cap == '' or device.caps.get(cap)))

            speed = self.opts['Custom Speed'][0]
            if speed.get_editable():
                import re
                max_write = device.caps.get('max_write')
                match = re.search(r'CD\s+(\d+)x, DVD\s+(\d+)x', max_write)
                try:
                    cdx, dvdx = map(int, match.groups(0))
                except ValueError:
                    raise
                else:
                    speed.set_range(1, cdx or 52)

        else:
            if self.device is device:
                self.device = None

    def on_verify_toggled(self, toggle, verify):
        if toggle.get_active():
            self.verify = verify
        else:
            if self.verify is verify:
                self.verify = None

    def on_option_toggled(self, toggle, name):
        self.opts[name][1] = toggle.get_active()

    def on_speed_toggled(self, toggle, custom):
        pass

    def on_speed_changed(self, spin):
        pass
