#
#    RedEye is a Disc Authoring and Burning program.
#    Copyright (C) 2003  Michael Urman
#
#    RedEye is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    RedEye is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with RedEye; if not, write to the Free Software Foundation, Inc., 59
#    Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

from RedEye.Helper import Helper

class DiscRecord(Helper.program):
    pass

class CDRecord(DiscRecord):

    device = None
    write_time = 0.0
    fixate_time = 0.0

    def __init__(self):
        super(CDRecord, self).__init__('cdrecord')

    params = [
        ('-dummy', 'test_burn', 0, False,
            "Go through the recording process, but with the laser off"),
        ('-dao', 'session_at_once', 0, False,
            "Set Session/Disc At Once mode; requires knowing track size"),
        ('-raw', 'raw_write', 0, False,
            "Enable raw writing; requires knowing track size"),
        ('-multi', 'multi_session', 0, False,
            "Fixate sessions to allow later sessions; may conflict with DAO"),
        ('-msinfo', 'msinfo', 0, False,
            "Used internally to retrieve Nth session info for mkisofs"),
        ('-fix', 'fixate', 0, False,
            "Fixate a previously written but unfixated disc"),
        ('-nofix', 'nofixate', 0, False,
            "Don't fixate after writing tracks; for step-by-step audio"),
        ('-waiti', 'wait_for_input', 0, False,
            "Wait for input before opening the SCSI driver; for Nth session"),
        ('-eject', 'eject', 0, False,
            "Eject the disc after the current operation"),
        ('-checkdrive', 'checkdrive', 0, False,
            "Check for existence, knowledge of drive; return 0 if so"),
        ('-scanbus', 'scanbus', 0, False,
            "Scan all devices on all SCSI buses and print -inq info"),
        ('-overburn', 'overburn', 0, False,
            "Allow burns over the official medium size.  May require DAO, or may fail"),
        ('-pad', 'pad', 0, True,
            "Pad tracks to allowed sizes"),
        ('-useinfo', 'useinfo', 0, False,
            "Use *.inf files to specify audio options"),
        ('-vv', 'verbose', 0, True,
            "Increase verbosity of cdrecord.  Risks underruns"),

        ('speed=%s', 'speed', 1, None,
            "Set the speed.  On recent MCC drives, defaults to max"),
        ('blank=%s', 'blanktype', 1, None,
            "Blank a RW; one of {help,all,fast,track,unreserve,trtail,unclose,session}"),
        ('dev=%s', 'device', 1, None,
            "Set the Writing device"),
        ('gracetime=%s', 'gracetime', 1, None,
            "Wait N seconds before starting to write.  Must be >= 2"),
        ('driver=%s', 'driver', 1, None,
            "Override the driver; see driver=help for details"),
        ('driveropts=%s', 'driveropts', 1, None,
            "Set specific driver options; list with driveropts=help -checkdrive"),
        ('mcn=%s', 'media_catalog_number', 1, None,
            "Set the Media Catalog Number"),
        ('cuefile=%s', 'cuefile', 1, None,
            "Take recording information from a CDRWIN CUE file; requires DAO"),
        ('tsize=%s', 'track_size', 1, None,
            "Specify the track size, for DAO/SAO burns")
    ]

    def command(self):
        if self.binary is None:
            raise ValueError('%s could not be found in your $PATH' % self.binary_name)
        args = [self.binary]

        for arg, attr, takes_arg, default, description in self.params:
            if takes_arg:
                if hasattr(self, attr):
                    args.append(arg % getattr(self, attr))
            else:
                if default:
                    args.append(arg)

        for track in self.tracks:
            args.extend(track.get_args())

        return tuple(args)

    def get_devices(self):
        devs = []
        for line in self.run_stdout('-scanbus'):
            if line.startswith('\t'):
                dev, desc = line.strip().split('\t')
                if "'" in desc:
                    dev = dev.strip()
                    idx, name, space1, info, space2, ser, desc = \
                            [ s.strip() for s in desc.split("'") ]
                    devs.append(Device(dev, '%s: %s' % (name, info)))
        return tuple(devs)

    def get_capabilities(self, dev):
        return self.run_stdout('dev=%s' % dev, '-prcap')

    def get_multisession_info(self):
        return self.run_stdout('dev=%s' % dev, '-msinfo')

class Device(object):

    def __init__(self, dev, description):
        self.dev = dev
        self.description = description
        self.caps = {}

        for line in CDRecord().get_capabilities(dev).readlines():
            if line == '':
                continue

            for match, cap in self.colon_capabilities:
                idx = line.find(match)
                if idx >= 0:
                    val = line[idx+len(match):]
                    val = val.strip(' :\n')
                    self.caps[cap] = val
                    break
            else:
                for match, cap in self.is_capabilities:
                    idx = line.find(match)
                    if idx >= 0:
                        if line.find('Is not') >= 0:
                            self.caps[cap] = False
                            break
                        elif line.find('Is') >= 0:
                            self.caps[cap] = True
                            break
                else:
                    for match, cap in self.does_capabilities:
                        idx = line.find(match)
                        if idx >= 0:
                            if line.find('Does not') >= 0:
                                self.caps[cap] = False
                                break
                            elif line.find('Does') >= 0:
                                self.caps[cap] = True
                                break

    does_capabilities = [
        ('deliver composite A/V data', 'delivers_composite_av'),

        ('read CD-R media', 'reads_cdr'),
        ('read CD-RW media', 'reads_cdrw'),
        ('read DVD-ROM media', 'reads_dvdrom'),
        ('read DVD-R media', 'reads_dvdr'),
        ('read DVD-RAM media', 'reads_dvdram'),
        ('read Mode 2 Form 1 blocks', 'reads_mode2form1'),
        ('read Mode 2 Form 2 blocks', 'reads_mode2form2'),
        ('read digital audio blocks', 'reads_digital_audio'),
        ('read multi-session CDs', 'reads_multisession'),
        ('read fixed-packet CD media using Method 2', 'reads_fixed_packet_2'),
        ('read CD bar code', 'reads_barcode'),
        ('read R-W subcode information', 'reads_rw_subcode'),
        ('read raw P-W subcode data from lead in', 'reads_pw_subcode'),

        ('write CD-R media', 'writes_cdr'),
        ('write CD-RW media', 'writes_cdrw'),
        ('write DVD-R media', 'writes_dvdr'),
        ('write DVD-RAM media', 'writes_dvdram'),

        ('allow media to be locked in the drive via PREVENT/ALLOW command', 'allows_lock_via_prevent'),
        ('have valid data on falling edge of clock', 'has_valid_on_falling'),
        ('have load-empty-slot-in-changer feature', 'has_changer_load_empty'),
        ('lock media on power up via prevent jumper', 'locks_via_prevent'),

        ('play audio CDs', 'plays_audio_cd'),
        ('restart non-streamed digital audio reads accurately', 'restarts_audio_accurately'),

        ('return R-W subcode de-interleaved and error-corrected', 'returns_rw_subcode_corrected'),
        ('return CD media catalog number', 'returns_cd_mcn'),
        ('return CD ISRC information', 'returns_isrc_info'),


        ('send digital data LSB-first', 'sends_data_LSB_first'),
        ('set LRCK high for left-channel data', 'sets_lrck_high_on_left'),

        ('support test writing', 'supports_test'),
        ('support Buffer-Underrun-Free recording', 'supports_burnfree'),
        ('support C2 error pointers', 'supports_c2_pointers'),
        ('support individual volume control setting for each channel', 'supports_channel_volume'),
        ('support independent mute setting for each channel', 'supports_channel_mute'),
        ('support digital output on port 1', 'supports_do_port1'),
        ('support digital output on port 2', 'supports_do_port2'),
        ('support ejection of CD via START/STOP command', 'supports_eject'),
        ('support changing side of disk', 'supports_side_change'),
        ('support Individual Disk Present feature', 'supports_disk_present'),
    ]
    is_capabilities = [ ('currently in a media-locked state', 'is_locked') ]
    colon_capabilities = [
        ('scsidev', 'scsidev'),
        ('Linux sg driver version', 'sg_driver_version'),
        ('Device type', 'device_type'),
        ('Version', 'device_version'),
        ('Response Format', 'response_format'),
        ('Vendor_info', 'vendor_info'),
        ('Identifikation', 'device_identification'),
        ('Revision', 'device_revision'),
        ('Device seems to be', 'device_seems'),
        ('Number of volume control levels', 'volume_control_levels'),
        ('Length of data in BCLKs', 'bclk_data_length'),
        ('Maximum read  speed', 'max_read'),
        ('Current read  speed', 'current_read'),
        ('Maximum write speed', 'max_write'),
        ('Current write speed', 'current_write'),
        ('Rotational control selected', 'rotational_control'),
        ('Buffer size in KB', 'buffer_size_kb'),
        ('Copy management revision supported', 'supported_copy_management'),
        ('Number of supported write speeds', 'supported_write_speeds'),
    ]


class Track(object):
    def __init__(self, filename):
        self.filename = filename

class AudioTrack(Track):
    def get_args(self):
        return '-audio', self.filename

class DataTrack(Track):
    def get_args(self):
        return '-data', self.filename
