#
#    RedEye is a Disc Authoring and Burning program.
#    Copyright (C) 2003  Michael Urman
#
#    RedEye is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    RedEye is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with RedEye; if not, write to the Free Software Foundation, Inc., 59
#    Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

import os, sys

class program(object):

    """Helper.program wraps a program and the ways you are likely to want to
    execute it.  Subclasses are encouraged to provide helper functions for
    common operations."""

    status = None
    done = False
    stderr_last = None
    binary = None

    def __init__(self, binary):
        self.binary_name = binary
        try: self.binary = self.findprogram(binary)
        except ValueError: pass

    def findprogram(self, name):
        for path in os.environ['PATH'].split(os.pathsep):
            bin = os.path.join(path, name)
            if os.path.exists(bin):
                # check for executable?  permissions?
                return bin
        raise ValueError("Can't find %s in $PATH" % name)

    def run_stdout(self, *args):
        if self.binary is None:
            raise ValueError('%s could not be found in your $PATH' % self.binary_name)

        if len(args) is 0:
            command = self.command()
        else:
            command = (self.binary,) + args

        #print command

        stdin, stdout, stderr = os.popen3(command)
        stdin.close()
        self.stderr_last = stderr.read()
        stderr.close()
        return stdout

    def command(self):
        raise NotImplementedError("Helper.program is an abstract class")

    def execute(self, stdin=None, stdout=None, stderr=None):
        if stdin is None:
            stdin = os.pipe()
        if stdout is None:
            stdout = os.pipe()
        if stderr is None:
            stderr = os.pipe()

        command = self.command()
        self.progpid = os.fork()
        if self.progpid == 0:
            # child - hook up file descriptors
            os.dup2(stdin[0], sys.stdin.fileno())
            os.dup2(stdout[1], sys.stdout.fileno())
            os.dup2(stderr[1], sys.stderr.fileno())

            for i in range(3, 256):
                try: os.close(i)
                except OSError: pass

            os.execvp(self.binary_name, cmd)

        # child has execvp'd, so only parent down from here
        os.close(stdin[0])
        try: self.stdin = os.fdopen(stdin[1], 'w')
        except OSError: pass

        os.close(stdout[1])
        try: self.stdout = os.fdopen(stdout[0], 'r')
        except OSError: pass

        os.close(stderr[1])
        try: self.stderr = os.fdopen(stderr[0], 'r')
        except OSError: pass

        self.stdout_buf = streambuf()
        self.stderr_buf = streambuf()

    def poll(self):
        """Check for return status of the execution.  Return None while still
        running."""
        if self.status is None:
            try:
                pid, status = os.waitpid(self.progpid, os.WNOHANG)
                if pid == self.progpid:
                    self.status = status
            except os.error:
                pass

        return self.status

    def wait(self):
        """Get (wait for) return status of the execution."""
        if self.status is None:
            pid, status = os.waitpid(self.progpid, 0)
            if pid == self.progpid:
                self.status = status
        return self.status

class streambuffer(object):
    """buffer reads and writes, returning a line at a time when available"""

    def __init__(self, newline='\r\n'):
        self.fulllines = []
        self.partial = []
        self.newline = newline

    def append(self, bytes):
        nl = self.newline
        self.partial.append(bytes)

        if nl in bytes:
            buffer = ''.join(self.partial)
            del self.buffer[:]
            self.fulllines.extend(buffer.split(nl))

            if not bytes.endswith(nl):
                self.buffer.append(self.fulllines.pop())

    def __iter__(self):
        while len(self.fulllines):
            yield self.fulllines.pop(0)

    
