#
#    RedEye is a Disc Authoring and Burning program.
#    Copyright (C) 2003  Michael Urman
#
#    RedEye is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    RedEye is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with RedEye; if not, write to the Free Software Foundation, Inc., 59
#    Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

import gtk, gobject, os, errno
from RedEye import Config
from RedEye.Application import RedEye
from RedEye.Layout import RowTypes


class VirtualDisc(RowTypes.Root):
    icon = 'disc'
    container_types = ()

class VirtualSession(RowTypes.Session):
    icon = 'disc'
    container_types = RowTypes.Root

class VirtualDataTrack(RowTypes.Track):
    icon = 'data_track'
    is_editable = True
    container_types = RowTypes.Session
    default_name = 'Data Track'

class VirtualAudioTrack(RowTypes.Track):
    icon = 'audio_track'
    default_name = 'Audio Track'

class VirtualFolder(RowTypes.Folder):
    icon = 'directory'
    is_editable = True
    default_name = 'Folder'

VirtualFolder.container_types = VirtualFolder, VirtualDataTrack

class VirtualFile(RowTypes.File):
    icon = 'data_file'
    container_types = VirtualFolder, VirtualDataTrack
    is_sized = True
    is_editable = True
    default_name = 'File'

    def __init__(self, name, view, source=None):
        super(VirtualFile, self).__init__(name, view, source)
        if source is not None:
            self.size = os.stat(source)[6]

class VirtualLayout(gtk.TreeView):

    """RedEye.Layout.Virtual.VirtualLayout

    Layout a virtual disc to be written"""

    column_model = [
        ("Edit", 0, gobject.TYPE_BOOLEAN),
        ("Icon", 1, gtk.gdk.Pixbuf),
        ("File", 2, gobject.TYPE_STRING),
        ("Size", 3, gobject.TYPE_UINT64),
        ("Type", 4, gobject.TYPE_STRING),
        ("Orig", 5, gobject.TYPE_STRING),
    ]

    column = dict([ (s,i) for s,i,t in column_model ])

    source_targets = [
        ('local_widget', gtk.TARGET_SAME_WIDGET, 0),
    ]
    dest_targets_yes = [
        ('local_widget', gtk.TARGET_SAME_WIDGET, 0),
        #('text/uri-list', 0, 8)
    ]
    dest_targets_no = dest_targets_yes[1:]

    def __init__(self):
        gtk.TreeView.__init__(self)
        self.get_selection().set_mode(gtk.SELECTION_MULTIPLE)

        self.model = gtk.TreeStore(gobject.TYPE_PYOBJECT, gobject.TYPE_BOOLEAN)
        self.set_model(self.model)
        #self.model.connect_after('row-inserted', self.on_row_inserted)
        #self.model.connect('row-deleted', self.on_row_deleted)
        self.recursive_add_roots = {}
        self.recursive_updated = {}

        def render_fileicon(column, cell, model, iter):
            rowobj = model.get_value(iter, 0)
            renderer = column.get_cell_renderers()[0]
            renderer.set_property('pixbuf', rowobj.icon)

        def render_filename(column, cell, model, iter):
            rowobj = model.get_value(iter, 0)
            renderer = column.get_cell_renderers()[1]
            renderer.set_property('text', rowobj.get_name())

        def render_filesize(column, cell, model, iter):
            rowobj = model.get_value(iter, 0)
            renderer = column.get_cell_renderers()[0]
            size = rowobj.get_size()
            if size == 0 and not rowobj.is_sized:
                sizestr = ''
            else:
                sizestr = Config.filesize(size)
                if rowobj.is_sized:
                    sizestr = sizestr.join('  ')
                else:
                    sizestr = sizestr.join('()')
            renderer.set_property('text', sizestr)

        # View: Icon+File
        col = gtk.TreeViewColumn()
        col.set_title('File')
        rp = gtk.CellRendererPixbuf()
        col.pack_start(rp, expand=False)
        rt = gtk.CellRendererText()
        col.pack_start(rt, expand=False)
        col.add_attribute(rt, 'editable', 1)
        rt.connect('edited', self.on_edit_filename)

        col.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
        col.set_cell_data_func(rp, render_fileicon)
        col.set_cell_data_func(rt, render_filename)

        self.append_column(col)
        self.set_expander_column(col)

        # View: Size
        col = gtk.TreeViewColumn()
        col.set_title('Size')
        rt = gtk.CellRendererText()
        rt.set_property('xalign', 1.0)
        col.pack_start(rt, expand=False)

        col.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
        col.set_cell_data_func(rt, render_filesize)
        self.append_column(col)

        self.connect('button-press-event', self.on_button_press)
        self.connect('key-press-event', self.on_key_press)
        #self.set_reorderable(True)

        self.append(VirtualDisc('Disc Layout', self), None)
        self.append_session()

        self.setup_dnd()

    def get_selected_iters(self):
        iters = []
        selection = self.get_selection()
        selection.selected_foreach(lambda m,p,it,its: its.append(it), iters)
        return iters


    def append(self, item, iter):
        if isinstance(item, type):
            item = item(item.default_name, self)
        item_iter = self.model.append(iter, row=(item, item.is_editable))
        item.model_iter = item_iter
        if iter is not None:
            self.expand_row(self.model.get_path(iter), False)
            parent = self.model.get_value(iter, 0)
            parent.children.append(item)
        if isinstance(item, VirtualFolder) and item.source is not None:
            self.recursive_add_roots[item.source] = item
            self.recursive_add_count = 0
            gtk.idle_add(self.recursive_add, os.walk(item.source))
        elif item.size > 0:
            self.add_size(item.size, iter)
        gtk.idle_add(self.grab_focus)
        gtk.idle_add(self.set_cursor, self.model.get_path(item_iter),
                self.get_column(0), item.is_editable)
        return item_iter

    def add_size(self, size, iter, update=True):
        """Add <size> (positive or negative) to <iter> and <iter>'s ancestors.
        Stop at Sessions, at least until we figure out a good method.
        Call model.row_changed() on each update unless <update> is False.
        Return a list of changed objects."""

        if size == 0: return ()
        get_path = self.model.get_path
        get_value = self.model.get_value
        iter_parent = self.model.iter_parent
        row_changed = self.model.row_changed
        changed = []
        while iter is not None:
            obj = get_value(iter, 0)
            if isinstance(obj, RowTypes.Session):
                break
            obj.size += size
            changed.append(obj)
            iter = iter_parent(iter)
        if update:
            for obj in changed:
                row_changed(get_path(obj.model_iter), obj.model_iter)
        return changed

    def recursive_add(self, pathiter, root=None, dirs=(), files=()):
        """"Recursively" add all the files under <pathiter> (as returned from
        os.walk() - it does all the actual recursion) in blocks of up to 20 at a
        time (to ensure program interactivity).  Delay add_size() updates to
        keep overhead lower."""

        process = 30 # add no more than 30 dirs and files per call
        processed = 0
        if len(dirs) + len(files) == 0:
            try:
                try: root, dirs, files = pathiter.next()
                except StopIteration:
                    self.statusbar.pop(1)
                    process = 0
                    return
                dirs.sort()
                files.sort()
                dirs = dirs[:]
                process /= 2 # do less work after a sort
                self.recursive_add_count += len(files) + len(dirs)
                self.statusbar.pop(1)
                self.statusbar.push(1, 'Read %d files...'
                        % self.recursive_add_count)
            finally:
                # always update at this point - either if we have at least 45
                # updates (to limit multiple updates of branch roots) or if
                # we're done and process is 0
                if len(self.recursive_updated) > process*3:
                    get_path = self.model.get_path
                    row_changed = self.model.row_changed
                    for obj in self.recursive_updated.itervalues():
                        objiter = obj.model_iter
                        if objiter is not None:
                            row_changed(get_path(objiter), objiter)
                    self.recursive_updated.clear()

        # a root can disappear in some valid circumstances involving the user
        # removing a parent path.  Trap it and go on, as it could be an
        # intermediate parent path instead of the root of this add.
        try:
            parent = self.recursive_add_roots[root]
            iter = parent.model_iter
        except KeyError:
            gtk.idle_add(self.recursive_add, pathiter)
            return

        # similarly, the "real" delete case will set a parent iter to None
        if iter is None:
            gtk.idle_add(self.recursive_add, pathiter)
            return

        append = self.model.append
        # Add up to <process> directories
        for dir in dirs[:process]:
            processed += 1
            source = os.path.join(root, dir)
            item = VirtualFolder(dir, self, source)
            item_iter = append(iter, row=(item, item.is_editable))
            item.model_iter = item_iter
            self.recursive_add_roots[source] = item
            parent.children.append(item)

        del dirs[:process]
        process -= processed
        processed = 0

        # If <process> wasn't exhausted yet, add up to <process> files
        for file in files[:process]:
            source = os.path.join(root, file)
            processed += 1
            try:
                item = VirtualFile(file, self, source)
            except OSError, err:
                if err.errno == errno.ENOENT:
                    try:
                        os.lstat(source)
                    except OSError: RedEye.messages.warn(
                            'Omitting nonexistant file: %s' % source)
                    else: RedEye.messages.warn(
                            'Omitting stale symlink: %s' % source)
                else:
                    RedEye.messages.error(
                        "Unexpected error while adding file: "+str(err))
                continue
            item_iter = append(iter, row=(item, item.is_editable))
            item.model_iter = item_iter
            parent.children.append(item)
            # track size updates for later view updates
            if item.size > 0:
                for obj in self.add_size(item.size, iter, False):
                    self.recursive_updated[id(obj)] = obj
        del files[:process]
        if len(files) + len(dirs) == 0:
            del self.recursive_add_roots[root]

        # The pseudo recursion - continue this call after gtk processes events
        gtk.idle_add(self.recursive_add, pathiter, root, dirs, files)

    def append_session(self, session=None):
        if session is None:
            session = VirtualSession('Session', self)
        root = self.model.get_iter_root()
        return self.append(session, root)

    def append_track(self, track=VirtualDataTrack, container=None):
        # try for the last present session
        if container is None:
            container = self.last_iter(VirtualSession)
        # failing that, create a session
        if container is None:
            container = self.append_session()
        return self.append(track, container)

    def append_folder(self, folder=VirtualFolder, container=None):
        # try for the last present data track
        if container is None:
            container = self.last_iter(VirtualDataTrack)
        # failing that, create a data track
        if container is None:
            container = self.append_track(VirtualDataTrack)
        return self.append(folder, container)

    def append_file(self, file=VirtualFile, container=None):
        # try for the last present data track
        if container is None:
            container = self.last_iter(VirtualDataTrack)
        # failing that, create a data track
        if container is None:
            container = self.append_track(VirtualDataTrack)
        return self.append(file, container)

    def last_iter(self, kind):
        """Return the bottom-most iter with an object of the specified kind(s)"""
        iters = []
        iter = self.model.get_iter_root()
        while iter is not None:
            iters.append(iter)
            iter = self.model.iter_next(iter)

        bottom = None
        while len(iters):
            iter = iters.pop()
            obj = self.model.get_value(iter, 0)
            if isinstance(obj, kind):
                bottom = iter
                del iters[:]  # nothing earlier could be it
            if self.model.iter_has_child(iter):
                iter = self.model.iter_children(iter)
                while iter is not None:
                    iters.append(iter)
                    iter = self.model.iter_next(iter)
        return bottom

    #def on_row_inserted(*args): pass 
    #def on_row_deleted(*args): pass
    def on_edit_filename(self, render, path, new):
        try:
            it = self.model.get_iter(path)
            obj = self.model.get_value(it, 0)
            obj.set_name(new)
            self.model.row_changed(path, it)
        except ValueError:
            pass

    def on_button_press(self, view, event):
        """Handle treeview context menu on RMB"""
        assert self is view
        if event.button == 3:
            x = int(event.x)
            y = int(event.y)
            time = gtk.get_current_event_time()
            try:
                path, col, cellx, celly = view.get_path_at_pos(x, y)
            except TypeError:
                # ignore clicks off of current items
                return False
            view.grab_focus()
            self.popup_path = path
            it = self.model.get_iter(path)
            rowobj = self.model.get_value(it, 0)
            if rowobj.popup is not None:
                view.set_cursor(path, col, False)
                rowobj.popup.popup(None, None, None, event.button, time)
                return True
        return False

    def on_key_press(self, view, event):
        """Handle <Delete>"""
        assert self is view
        if gtk.gdk.keyval_name(event.keyval) == 'Delete':
            self.remove_selected()


    def describe(self):
        root = self.model.get_iter_root()
        obj = self.model.get_value(root, 0)
        __import__('pprint').pprint(obj.describe(self.progeny(root)))

    def progeny(self, iter):
        if iter is None:
            raise StopIteration()

        child = self.model.iter_children(iter)
        while child is not None:
            childobj = self.model.get_value(child, 0)
            yield childobj, self.progeny(child)
            child = self.model.iter_next(child)

    def remove_selected(self):
        iters = self.get_selected_iters()

        try:
            path = self.model.get_path(iters[0])
        except IndexError:
            return

        def get_progeny(model, iter):
            progeny = [iter]
            if model.iter_has_child(iter):
                child = model.iter_children(iter)
                while child is not None:
                    progeny.extend(get_progeny(model, child))
                    child = model.iter_next(child)
            return progeny

        iter_parent = self.model.iter_parent
        # model.remove() removes progeny uncleanly; so do it manually
        while len(iters):
            iter = iters.pop()
            progeny = get_progeny(self.model, iter)
            progeny.reverse()
            for iter in progeny:
                # remove things only once
                if iter in iters: iters.remove(iter)
                # don't remove unremovable things
                obj = self.model.get_value(iter, 0)
                if obj.is_removable:
                    if obj.size > 0 and iter_parent(iter) not in iters:
                        self.add_size(-obj.size, iter)
                    del obj.model_iter
                    self.model.remove(iter)

        # try to focus something sane
        while len(path):
            try:
                self.model.get_iter(path)
            except ValueError:
                if path[-1] > 0:
                    path = path[:-1] + (path[-1]-1,)
                else:
                    path = path[:-1]
            else:
                self.set_cursor(path, None, False)
                break

    def remove_selected(self):
        iters = []
        selection = self.get_selection()
        selection.selected_foreach(lambda m,p,it,its: its.append(it), iters)
        get_value = self.model.get_value

        objs = [get_value(it, 0) for it in iters]

        try:
            path = self.model.get_path(iters[0])
        except IndexError:
            return

        gtk.idle_add(self.remove_objects, objs, path)

    def remove_objects(self, objects, path):
        #process = 20
        #while process > 0 and len(objects):
        while len(objects):
            obj = objects.pop() # take an object
            if not obj.is_removable:
                continue
            iter = obj.model_iter
            # if it's still in the tree
            if iter is not None:
                # remove from its parent
                parent_iter = self.model.iter_parent(iter)
                parent = self.model.get_value(parent_iter, 0)
                try: parent.children.remove(obj)
                except ValueError: continue
                # remove from the tree and adjust size
                del obj.model_iter
                self.model.remove(iter)
                self.add_size(-obj.size, parent_iter)
            # tell its children they're not in the tree
            for child in obj.children:
                del child.model_iter
            # recurse to all progeny
            objects.extend(obj.children)
            del obj.children[:]

        if len(objects):
            gtk.idle_add(self.remove_objects, objs, path)
        else:
            # try to focus something sane
            while len(path):
                try: self.model.get_iter(path)
                except ValueError:
                    if path[-1] > 0: path = path[:-1] + (path[-1]-1,)
                    else: path = path[:-1]
                else:
                    self.set_cursor(path, None, False)
                    break

    def copy_row(self, source, target, drop_position):
        """Copy tree model rows from <source> (iter) into, before, or after
        <target> (iter)

        All progeny of the source are copied, and expanded/collapsed remains"""

        model = self.model
        srow = model[source]
        if drop_position == gtk.TREE_VIEW_DROP_INTO_OR_BEFORE:
            new = model.prepend(parent=target, row=srow)
        elif drop_position == gtk.TREE_VIEW_DROP_INTO_OR_AFTER:
            new = model.append(parent=target, row=srow)
        elif drop_position == gtk.TREE_VIEW_DROP_BEFORE:
            new = model.insert_before(parent=None, sibling=target, row=srow)
        elif drop_position == gtk.TREE_VIEW_DROP_AFTER:
            new = model.insert_after(parent=None, sibling=target, row=srow)

        # copy children
        for i in range(model.iter_n_children(source)):
            child = model.iter_nth_child(source, i)
            self.copy_row(child, new, gtk.TREE_VIEW_DROP_INTO_OR_AFTER)

        was_expanded = self.row_expanded(model.get_path(source))
        if was_expanded:
            self.expand_row(model.get_path(source), False)

        # update internal iter ref
        srow[0].model_iter = new
        return new


    def check_drop(self, x, y):
        """Verify legality of a drop and return precalculated information"""

        # get target and drop style
        try: target_path, drop_pos = self.get_dest_row_at_pos(x, y)
        except TypeError: return None
        model = self.model
        # get presumed source - XXX
        iters = self.get_selected_iters()
        source = iters[0]

        target = model.get_iter(target_path)
        obj = model[source][0]
        # see if the _into_ drop is okay
        if drop_pos in (gtk.TREE_VIEW_DROP_INTO_OR_BEFORE,
                gtk.TREE_VIEW_DROP_INTO_OR_AFTER):
            container = model[target][0]
            # if _into_or_x fails, try _x
            if not self.drop_ok(obj, target, container):
                if drop_pos == gtk.TREE_VIEW_DROP_INTO_OR_BEFORE:
                    drop_pos = gtk.TREE_VIEW_DROP_BEFORE
                else:
                    drop_pos = gtk.TREE_VIEW_DROP_AFTER
                container = model.iter_parent(target)
                if container is not None:
                    container = model[container][0]
                if not self.drop_ok(obj, target, container):
                    return None
        # see if the _before or _after drop is okay
        else:
            container = model.iter_parent(target)
            if container is not None:
                container = model[container][0]
            if not self.drop_ok(obj, target, container):
                return None

        return obj, target, target_path, drop_pos


    def on_drag_data_received(self, view, ctx, x, y, data, info, etime):
        """Moves dropped TreeView rows."""
        ok = self.check_drop(x, y)
        if not ok:
            ctx.finish(success=False, del_=False, time=etime)
            return
        obj, target, target_path, drop_pos = ok
        source = obj.model_iter
        model = self.model
        oldparent = model.iter_parent(source)

        new = self.copy_row(source, target, drop_pos)
        # expand newly created parents
        if drop_pos in (gtk.TREE_VIEW_DROP_INTO_OR_BEFORE,
                gtk.TREE_VIEW_DROP_INTO_OR_AFTER):
            view.expand_row(target_path, False)
        ctx.finish(success=True, del_=True, time=etime)

        # update sizes, interal references
        newparent = model.iter_parent(new)
        if model.get_path(oldparent) != model.get_path(newparent):
            self.add_size(-obj.size, oldparent)
            self.add_size(obj.size, newparent)
            model.get_value(oldparent, 0).children.remove(obj)
            model.get_value(newparent, 0).children.append(obj)

    last_drag = None
    def on_drag_motion(self, view, ctx, x, y, etime):
        ok = self.check_drop(x, y)
        if ok:
            #ctx.drop_reply(True, etime)
            if self.last_drag is not True:
                self.enable_model_drag_dest(self.dest_targets_yes,
                        gtk.gdk.ACTION_MOVE)
                self.last_drag = True
        else:
            if self.last_drag is not False:
                self.last_drag = False
                self.enable_model_drag_dest(self.dest_targets_no,
                        gtk.gdk.ACTION_MOVE)

    def on_drag_end(self, *args):
        del self.last_drag

    def row_drop_possible(self, *args):
        print 'r_d_p'
        return gtk.TreeView.row_drop_possible(self, *args)

    def drop_ok(self, obj, target, container):
        model = self.model
        get_path = model.get_path
        return obj.can_go_in(container) \
            and get_path(obj.model_iter) != get_path(target) \
            and not model.is_ancestor(obj.model_iter, target)

    def setup_dnd(self):
        entries = [('example', gtk.TARGET_SAME_WIDGET, 0)]
        self.enable_model_drag_source(
                gtk.gdk.BUTTON1_MASK, self.source_targets, gtk.gdk.ACTION_MOVE)
        self.enable_model_drag_dest(self.dest_targets_yes, gtk.gdk.ACTION_MOVE)
        self.connect('drag-data-received', self.on_drag_data_received)
        self.connect('drag-motion', self.on_drag_motion)
        #self.connect('drag-drop', print_, 'drag_DROP')
        #self.connect('drag-begin', print_, 'drag_BEGIN')
        #self.connect('drag-data-delete', print_, 'drag_data_DELETE')
        #self.connect('drag-data-get', print_, 'drag_data_GET')
        self.connect('drag-end', self.on_drag_end)
        #self.connect('drag-leave', print_, 'drag_data_LEAVE')

def print_(*args):
    print args[-1]
